Program Ex02;
{ Example program demonstrating the loading of a 3d object and       }
{ spinning it on the screen using multiple colored polygons gouraud  }
{ shaded with multiple dynamic light sources.                        }

Uses
  Types,   { Inertia type declarations     }
  Inertia, { the 3d engine                 }
  Mode13h, { Basic Mode 13h graphics unit  }
  Crt;

Var
  View : ViewObject;   { The View System }
  Obj  : VectorObject; { A vector object }
  i    : Integer;

Begin
  InitInertia;

  If (LoadPalette('COPPRENV.PAL', 0) <> I_OK)       { load the palette }
    Then Begin
      Writeln('Error loading Palette.');
      Halt(0);
    End;
  If (LoadIntensityTable('COPPRENV.IT', 0) <> I_OK) { load intensity table }
    Then Begin
      Writeln('Error loading Intensity table.');
      Halt(0);
    End;
  If (LoadGVO(Obj, 'TORUS.GVO', 0) <> I_OK)         { load the vector object }
    Then Begin
      Writeln('Error loading TORUS object.');
      Halt(0);
    End;

  InitView(View);   { init the view }

  SetMode($13);              { Set graphics mode }
  SetPalette(GlobalPalette); { Set the palette }

  SetAbsoluteLocation(Obj, 0, 0, Obj.Radius * 2); { Set the object's location }
  PushLightSource(0, 0, -1);  { Add a light source }
  PushLightSource(1, 0, 0);   { Add another light source }

  SetRenderMask(Obj, SMOOTH or GOURAUD);    { set smooth surface }

  For i := 0 to (Obj.NumPoly - 1) do
    SetPolygonColor(Obj, i, 36 + (i And 1)*36);   { set the polygon colors }

  Repeat
    SetDeltaRotation(Obj, 0.01, 0.02, 0.03);  { rotate the object  }
    AddtoRenderList(View, Obj);      { add it to the view }
    Render(VirtualPage, 320, 200);       { render to the virtual page }
    Flip;                            { Copy the page over }
    RotateLightSource(0, 0.01, 0.02, 0.03); { rotate the first light source }
    RotateLightSource(1, 0.01, 0, 0.02); { rotate the first light source }
  Until KeyPressed;

  FreeView(View);         { Free the view's allocated memory }
  FreeVectorObject(Obj);  { free the vector object's allocated memory }
  FreeIntensityTable;     { free the intensity table from memory }

  SetMode($03);           { return to text mode }

  ShutdownInertia;
End.
